<?php

namespace Drupal\social_simple;

use Drupal\Core\Controller\TitleResolver;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Render\Renderer;

/**
 * Class SocialSimpleGenerator.
 *
 * @package Drupal\social_simple
 */
class SocialSimpleGenerator implements SocialSimpleGeneratorInterface {

  use StringTranslationTrait;

  /**
   * Drupal\Core\Controller\TitleResolver definition.
   *
   * @var \Drupal\Core\Controller\TitleResolver
   */
  protected $titleResolver;

  /**
   * Drupal\Core\Routing\CurrentRouteMatch definition.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Symfony\Component\HttpFoundation\RequestStack definition.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Drupal\Core\Config\ConfigFactory definition.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Drupal\Core\Render\Renderer .
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Constructs a new SocialSimpleGenerator object.
   *
   * @param \Drupal\Core\Controller\TitleResolver $title_resolver
   *   The title resolver service.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $current_route_match
   *   The current route match.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Render\Renderer $renderer_service
   *   The renderer service.
   */
  public function __construct(TitleResolver $title_resolver, CurrentRouteMatch $current_route_match, RequestStack $request_stack, ConfigFactory $config_factory, Renderer $renderer_service) {
    $this->titleResolver = $title_resolver;
    $this->currentRouteMatch = $current_route_match;
    $this->requestStack = $request_stack;
    $this->configFactory = $config_factory;
    $this->renderer = $renderer_service;
  }

  /**
   * {@inheritdoc}
   */
  public function buildSocialLinks(array $networks, $title, EntityInterface $entity = NULL) {
    $links = $this->generateSocialLinks($networks, $entity);

    $build = [
      '#theme' => 'social_simple_buttons',
      '#links' => $links,
      '#attributes' => [
        'class' => ['links', 'inline', 'social-buttons-links'],
      ],
      '#heading' => [
        'text' => $title,
        'level' => 'div',
        'attributes' => [
          'class' => ['social-buttons-title'],
        ],
      ],
    ];
    $build['#attached']['library'][] = 'social_simple/buttons';

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function generateSocialLinks(array $networks, EntityInterface $entity = NULL) {
    $links = [];
    $title = $this->getTitle($entity);
    $share_url = $this->getShareUrl($entity);

    foreach ($networks as $network_id => $network_name) {
      $links[$network_id] = $this->generateSocialLink($network_id, $network_name, $title, $share_url);
    }
    return $links;
  }

  /**
   * {@inheritdoc}
   */
  public function generateSocialLink($network_id, $network_name, $title, $share_url) {
    $attributes = $this->getLinkAttributes($network_name);
    switch ($network_id) {
      case 'twitter':
        $options = [
          'query' => [
            'url' => $share_url,
            'text' => $title,
          ],
          'absolute' => TRUE,
          'external' => TRUE,
        ];
        $url = Url::fromUri('https://twitter.com/intent/tweet/', $options);
        $link = [
          'url' => $url,
          'title' => ['#markup' => '<i class="fa fa-twitter"></i><span class="visually-hidden">' . $network_name . '</span>'],
          'attributes' => $attributes,
        ];
        break;

      case 'googleplus':
        $options = [
          'query' => [
            'url' => $share_url,
          ],
          'absolute' => TRUE,
          'external' => TRUE,
        ];
        $url = Url::fromUri('https://plus.google.com/share', $options);
        $link = [
          'url' => $url,
          'title' => ['#markup' => '<i class="fa fa-google-plus"></i><span class="visually-hidden">' . $network_name . '</span>'],
          'attributes' => $attributes,
        ];
        break;

      case 'facebook':
        $options = [
          'query' => [
            'u' => $share_url,
          ],
          'absolute' => TRUE,
          'external' => TRUE,
        ];
        $url = Url::fromUri('https://www.facebook.com/sharer/sharer.php', $options);
        $link = [
          'url' => $url,
          'title' => ['#markup' => '<i class="fa fa-facebook"></i><span class="visually-hidden">' . $network_name . '</span>'],
          'attributes' => $attributes,
        ];
        break;

      case 'linkedin':
        $options = [
          'query' => [
            'mini' => 'true',
            'url' => $share_url,
            'title' => $title,
          ],
          'absolute' => TRUE,
          'external' => TRUE,
        ];
        $url = Url::fromUri('https://www.linkedin.com/shareArticle', $options);
        $link = [
          'url' => $url,
          'title' => ['#markup' => '<i class="fa fa-linkedin"></i><span class="visually-hidden">' . $network_name . '</span>'],
          'attributes' => $attributes,
        ];
        break;

      default:
        $link = [];
        break;
    }

    return $link;

  }

  /**
   * {@inheritdoc}
   */
  public function getTitle(EntityInterface $entity = NULL) {
    if ($entity) {
      $title = $entity->label();
    }
    else {
      $title = $this->titleResolver->getTitle($this->requestStack->getCurrentRequest(), $this->currentRouteMatch->getRouteObject());
    }

    return is_string($title) ? $title : $this->renderer->render($title);
  }

  /**
   * {@inheritdoc}
   */
  public function getShareUrl(EntityInterface $entity = NULL) {
    if ($entity) {
      $share_url = $entity->toUrl('canonical', ['absolute' => TRUE])->toString();
    }
    else {
      $share_url = Url::fromRoute('<current>', [], ['absolute' => 'true'])->toString();
    }

    return $share_url;
  }

  /**
 * {@inheritdoc}
 */
  public function getLinkAttributes($network_name) {
    $attributes = [
      'data-popup-width' => 600,
      'data-popup-height' => 300,
      'data-toggle' => 'tooltip',
      'data-placement' => 'top',
      'title' => $network_name,
    ];
    return $attributes;
  }

  /**
   * {@inheritdoc}
   */
  public function getNetworks() {
    return [
      'twitter' => $this->t('Twitter'),
      'googleplus' => $this->t('Google+'),
      'facebook' => $this->t('Facebook'),
      'linkedin' => $this->t('Linked in'),
    ];
  }

}
